'''
Module with tests for BkkChecker class
'''
from importlib.resources import files

import pytest
import yaml

from ap_utilities.logging.log_store       import LogStore
from ap_utilities.bookkeeping.bkk_checker import BkkChecker

log = LogStore.add_logger('ap_utilities:tests:test_bkk_check')
# ----------------------------------------
def _sections_from_path( path : str) -> dict[str, dict]:
    with open(path, encoding='utf-8') as ifile:
        d_cfg = yaml.safe_load(ifile)

    return d_cfg
# ----------------------------------------
@pytest.fixture(scope='session', autouse=True)
def _initialize():
    LogStore.set_level('ap_utilities:Bookkeeping.bkk_checker', 10)
# ----------------------------------------
def test_simple():
    '''
    Will save list of samples to YAML
    '''
    samples_path = files('ap_utilities_data').joinpath('rd_samples.yaml')
    samples_path = str(samples_path)

    d_cfg      = _sections_from_path(samples_path)
    d_sections = d_cfg['sections']
    for name, d_section in d_sections.items():
        log.info(f'Processing section: {name}')
        obj=BkkChecker(name, d_section)
        obj.save()
# ----------------------------------------
def test_nick_evt():
    '''
    Will test reading when there are both evt_type and nickname sections 
    '''
    samples_path = files('ap_utilities_data').joinpath('nick_evt.yaml')
    samples_path = str(samples_path)

    d_cfg        = _sections_from_path(samples_path)
    d_sections   = d_cfg['sections']
    for name, d_section in d_sections.items():
        log.info(f'Processing section: {name}')
        obj=BkkChecker(name, d_section)
        obj.save()
# ----------------------------------------
def test_multithreaded():
    '''
    Will save list of samples to YAML using 4 threads
    '''
    samples_path = files('ap_utilities_data').joinpath('rd_samples.yaml')
    samples_path = str(samples_path)

    d_cfg = _sections_from_path(samples_path)
    d_sections = d_cfg['sections']
    for name, d_section in d_sections.items():
        log.info(f'Processing section: {name}')
        obj=BkkChecker(name, d_section)
        obj.save(nthreads=8)
# ----------------------------------------



'''
Script that will be used to update the mapping between event type and
formatted sample names
'''
from importlib.resources import files

import yaml

from ap_utilities.decays import utilities as aput

# --------------------------------
def _add_data_low_org(d_low_org : dict[str,str]) -> dict[str,str]:
    d_low_org['data_24_magdown_24c1'] = 'DATA_24_MagDown_24c1'
    d_low_org['data_24_magdown_24c2'] = 'DATA_24_MagDown_24c2'
    d_low_org['data_24_magdown_24c3'] = 'DATA_24_MagDown_24c3'
    d_low_org['data_24_magdown_24c4'] = 'DATA_24_MagDown_24c4'
    d_low_org['data_24_magup_24c1'  ] = 'DATA_24_MagUp_24c1'
    d_low_org['data_24_magup_24c2'  ] = 'DATA_24_MagUp_24c2'
    d_low_org['data_24_magup_24c3'  ] = 'DATA_24_MagUp_24c3'
    d_low_org['data_24_magup_24c4'  ] = 'DATA_24_MagUp_24c4'

    return d_low_org
# --------------------------------
def _load_file(file_name : str) -> dict[str,str]:
    file_path = files('ap_utilities_data').joinpath(file_name)
    file_path = str(file_path)

    with open(file_path, encoding='utf-8') as ifile:
        d_data = yaml.safe_load(ifile)

    return d_data
# --------------------------------
def _reformat(d_data : dict[str,str], keys : bool) -> dict[str,str]:
    if keys:
        d_data_form = { aput.format_nickname(key) : value for key, value in d_data.items() }
    else:
        d_data_form = { key : aput.format_nickname(value) for key, value in d_data.items() }

    return d_data_form
# --------------------------------
def _save(d_data : dict[str,str], file_name : str) -> None:
    file_path = files('ap_utilities_data').joinpath(file_name)
    file_path = str(file_path)

    with open(file_path, 'w', encoding='utf-8') as ofile:
        yaml.safe_dump(d_data, ofile)
# --------------------------------
def main():
    '''
    Starts here
    '''

    d_name_evt = _load_file('name_evt.yaml')
    d_evt_name = _load_file('evt_name.yaml')

    d_form_evt = _reformat(d_name_evt, keys=True)
    d_evt_form = _reformat(d_evt_name, keys=False)
    d_low_org  = { sample.lower() : sample for sample in d_form_evt }
    d_low_org  = _add_data_low_org(d_low_org)

    _save(d_low_org , 'lower_original.yaml')
    _save(d_form_evt,       'form_evt.yaml')
    _save(d_evt_form,       'evt_form.yaml')
# --------------------------------
if __name__ == '__main__':
    main()
